
# Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##############################################################################

#-----------------------------------------------------------------------
# This macro searches for Xlib and when it finds it it adds the 
# appropriate flags to CXXFLAGS and export the link sequence to 
# the variable XLIB. 
# In your configure.in file add:
#   LF_PATH_XLIB
# In your Makefile.am add
#   program_LDADD = .... $(XLIB)
#------------------------------------------------------------------------

AC_DEFUN(LF_PATH_XLIB,[
  AC_PATH_XTRA
  CXXFLAGS="$CXXFLAGS $X_CFLAGS"
  XLIB="$X_LIBS $X_PRE_LIBS -lX11 $X_EXTRA_LIBS"
  AC_SUBST(XLIB)
])

# ---------------------------------------------------------------------------
# This macro will set you up to link a program with the Tcl library:
# 1. Find out about linking the math library
# 2. See if we need to link in -ldl (it's needed on my Linux machine)
# 3. Find out the most recent version of Tcl
# In your configure.in file add:
#   LF_CONFIG_TCL
# and in your Makefile.am add
#   program_LDADD = .... @TCL_LIBS@
# which will expand to the Right Thing
# ---------------------------------------------------------------------------

AC_DEFUN(LF_CONFIG_TCL,[
  dnl Make sure to link the math library
  TCL_LIBS="${MATH_LIBS} ${DL_LIBS}"

if eval "test \"`echo '$''{'ac_cv_lib_socket_connect'+set}'`\" = set"; then
TCL_LIBS="${TCL_LIBS} -lsocket"
fi

if eval "test \"`echo '$''{'ac_cv_lib_nsl_gethostbyname'+set}'`\" = set"; then
TCL_LIBS="${TCL_LIBS} -lnsl"
fi

  dnl Search for the right Tcl library
  lf_found_tcl_lib=no
  for lf_lib in tcl8.0 tcl80 tcl7.6 tcl76 tcl7.5 tcl75 tcl7.4 tcl74 tcl
  do
    if test "$lf_found_tcl_lib" = no
    then
      AC_CHECK_LIB($lf_lib, Tcl_Init, 
                   [ TCL_LIBS="-l$lf_lib ${TCL_LIBS}"
                     lf_found_tcl_lib=yes ],
		    [],
		    ${TCL_LIBS})
    fi
  done

  dnl Now search for the Tcl private includes

  CY_AC_PATH_TCLH
  
  dnl Export the TCL_LIBS
  AC_SUBST(TCL_LIBS)
])

# -------------------------------------------------------------------------
# This macro will set you up to include headers for a program with the Tcl
# private header files:
# In your configure.in file add:
#   LF_CONFIG_TCL
# This is automatically called from LF_CONFIG_TCL, so you don't need
# to do anything special here.
# and in your Makefile.am add
#   program_LDADD = .... @TK_LIBS@ @TCL_LIBS@
# which will expand to the right thing
# This routine was taken from aclocal.m4 in the Expect 5.25 distribution.
# --------------------------------------------------------------------------

AC_DEFUN(CY_AC_PATH_TCLH,[
#
# Ok, lets find the tcl source trees so we can use the headers
# Warning: transition of version 9 to 10 will break this algorithm
# because 10 sorts before 9. We also look for just tcl. We have to
# be careful that we don't match stuff like tclX by accident.
# the alternative search directory is involked by --with-tclinclude
#
no_tcl=true
AC_MSG_CHECKING(for Tcl private headers)
AC_ARG_WITH(tclinclude, [  --with-tclinclude       directory where tcl private headers are], with_tclinclude=${withval})
AC_CACHE_VAL(ac_cv_c_tclh,[
# first check to see if --with-tclinclude was specified
if test x"${with_tclinclude}" != x ; then
  if test -f ${with_tclinclude}/tclInt.h ; then
    ac_cv_c_tclh=`(cd ${with_tclinclude}; pwd)`
  elif test -f ${with_tclinclude}/generic/tclInt.h ; then
    ac_cv_c_tclh=`(cd ${with_tclinclude}/generic; pwd)`
  else
    AC_MSG_ERROR([${with_tclinclude} directory doesn't contain private headers])
  fi
fi

# next check if it came with Tcl configuration file
if test x"${ac_cv_c_tclconfig}" != x ; then
  if test -f $ac_cv_c_tclconfig/../generic/tclInt.h ; then
    ac_cv_c_tclh=`(cd $ac_cv_c_tclconfig/../generic; pwd)`
  fi
fi

# next check in private source directory
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_c_tclh}" = x ; then
  for i in \
                ${srcdir}/../tcl \
                `ls -dr ${srcdir}/../tcl[[7-9]].[[0-9]] 2>/dev/null` \
                ${srcdir}/../../tcl \
                `ls -dr ${srcdir}/../../tcl[[7-9]].[[0-9]] 2>/dev/null` \
                ${srcdir}/../../../tcl \
                `ls -dr ${srcdir}/../../../tcl[[7-9]].[[0-9]] 2>/dev/null ` ; do
    if test -f $i/generic/tclInt.h ; then
      ac_cv_c_tclh=`(cd $i/generic; pwd)`
      break
    fi
  done
fi
# finally check in a few common install locations
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_c_tclh}" = x ; then
  for i in \
                `ls -dr /usr/local/src/tcl[[7-9]].[[0-9]] 2>/dev/null` \
                `ls -dr /usr/local/lib/tcl[[7-9]].[[0-9]] 2>/dev/null` \
                `ls -dr /usr/local/tcl[[7-9]].[[0-9]] 2>/dev/null` \
                /usr/local/src/tcl \
                /usr/local/lib/tcl \
                ${prefix}/include ; do
    if test -f $i/generic/tclInt.h ; then
      ac_cv_c_tclh=`(cd $i/generic; pwd)`
      break
    fi
  done
fi
# see if one is installed
if test x"${ac_cv_c_tclh}" = x ; then
   AC_HEADER_CHECK(tclInt.h, ac_cv_c_tclh=installed, ac_cv_c_tclh="")
fi
])
if test x"${ac_cv_c_tclh}" = x ; then
  TCLHDIR="# no Tcl private headers found"
  TCLHDIRDASHI="# no Tcl private headers found"
  AC_MSG_ERROR([Can't find Tcl private headers])
fi
if test x"${ac_cv_c_tclh}" != x ; then
  no_tcl=""
  if test x"${ac_cv_c_tclh}" = x"installed" ; then
    AC_MSG_RESULT([is installed])
    TCLHDIR=""
    TCLHDIRDASHI=""
    TCL_LIBRARY=""
  else
    AC_MSG_RESULT([found in ${ac_cv_c_tclh}])
    # this hack is cause the TCLHDIR won't print if there is a "-I" in it.
    TCLHDIR="${ac_cv_c_tclh}"
    TCLHDIRDASHI="-I${ac_cv_c_tclh}"
    TCL_LIBRARY=`echo $TCLHDIR | sed -e 's/generic//'`library
  fi
fi

AC_SUBST(TCLHDIR)
AC_SUBST(TCLHDIRDASHI)
AC_SUBST(TCL_LIBRARY)
])

# -------------------------------------------------------------------------
# This macro will set you up to link a program with the Tk library:
# 1. Find out what we need to do to link in X11
# 2. Find the most recent version of the Tk library
# In your configure.in file add:
#   LF_CONFIG_TK
# and in your Makefile.am add
#   program_LDADD = .... @TK_LIBS@ @TCL_LIBS@
# which will expand to the right thing
# --------------------------------------------------------------------------

AC_DEFUN(LF_CONFIG_TK,[
  
  dnl Find out what we need to link in X11 stuff
  CXXFLAGS="${CXXFLAGS} $X_CFLAGS"
  TK_LIBS="${X_LIBS} ${X_PRE_LIBS} -lX11 ${X_EXTRA_LIBS} ${TCL_LIBS}"

  lf_found_tk_lib=no
  for lf_lib in tk8.0 tk80 tk4.2 tk42 tk4.1 tk41 tk
  do
    if test $lf_found_tk_lib = no
    then
      AC_CHECK_LIB($lf_lib, Tk_Init,
                   [ TK_LIBS="-l${lf_lib} ${TK_LIBS}"
                     lf_found_tk_lib=yes ],
		     [],
		     ${TK_LIBS})
    fi
  done

  dnl Check for the Tk private header files.

  CY_AC_PATH_TKH
  
  dnl Export the TK_LIBS
  AC_SUBST(TK_LIBS)
])

# -------------------------------------------------------------------------
# This macro will set you up to include headers for a program with the Tk
# private header files:
# In your configure.in file add:
#   LF_CONFIG_TK
# This is automatically called from LF_CONFIG_TK, so you don't need
# to do anything special here.
# and in your Makefile.am add
#   program_LDADD = .... @TK_LIBS@ @TCL_LIBS@
# which will expand to the right thing
# This routine was taken from aclocal.m4 in the Expect 5.25 distribution.
# --------------------------------------------------------------------------
AC_DEFUN(CY_AC_PATH_TKH, [
#
# Ok, lets find the tk source trees so we can use the headers
# If the directory (presumably symlink) named "tk" exists, use that one
# in preference to any others.  Same logic is used when choosing library
# and again with Tcl. The search order is the best place to look first, then in
# decreasing significance. The loop breaks if the trigger file is found.
# Note the gross little conversion here of srcdir by cd'ing to the found
# directory. This converts the path from a relative to an absolute, so
# recursive cache variables for the path will work right. We check all
# the possible paths in one loop rather than many seperate loops to speed
# things up.
# the alternative search directory is involked by --with-tkinclude
#
#no_tk=true
AC_MSG_CHECKING(for Tk private headers)
AC_ARG_WITH(tkinclude, [  --with-tkinclude       directory where tk private headers are], with_tkinclude=${withval})
AC_CACHE_VAL(ac_cv_c_tkh,[
# first check to see if --with-tkinclude was specified
if test x"${with_tkinclude}" != x ; then
  if test -f ${with_tkinclude}/tk.h ; then
    ac_cv_c_tkh=`(cd ${with_tkinclude}; pwd)`
  elif test -f ${with_tkinclude}/generic/tk.h ; then
    ac_cv_c_tkh=`(cd ${with_tkinclude}/generic; pwd)`
  else
    AC_MSG_ERROR([${with_tkinclude} directory doesn't contain private headers])
  fi
fi

# next check if it came with Tk configuration file
if test x"${ac_cv_c_tkconfig}" != x ; then
  if test -f $ac_cv_c_tkconfig/../generic/tk.h ; then
    ac_cv_c_tkh=`(cd $ac_cv_c_tkconfig/../generic; pwd)`
  fi
fi

# next check in private source directory
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_c_tkh}" = x ; then
  for i in \
                ${srcdir}/../tk \
                `ls -dr ${srcdir}/../tk[[4-9]].[[0-9]] 2>/dev/null` \
                ${srcdir}/../../tk \
                `ls -dr ${srcdir}/../../tk[[4-9]].[[0-9]] 2>/dev/null` \
                ${srcdir}/../../../tk \
                `ls -dr ${srcdir}/../../../tk[[4-9]].[[0-9]] 2>/dev/null ` ; do
    if test -f $i/generic/tk.h ; then
      ac_cv_c_tkh=`(cd $i/generic; pwd)`
      break
    fi
  done
fi
# finally check in a few common install locations
#
# since ls returns lowest version numbers first, reverse its output
if test x"${ac_cv_c_tkh}" = x ; then
  for i in \
                `ls -dr /usr/local/src/tk[[4-9]].[[0-9]] 2>/dev/null` \
                `ls -dr /usr/local/lib/tk[[4-9]].[[0-9]] 2>/dev/null` \
                `ls -dr /usr/local/tk[[4-9]].[[0-9]] 2>/dev/null` \
                /usr/local/src/tk \
                /usr/local/lib/tk \
                ${prefix}/include ; do
    if test -f $i/generic/tk.h ; then
      ac_cv_c_tkh=`(cd $i/generic; pwd)`
      break
    fi
  done
fi
# see if one is installed
if test x"${ac_cv_c_tkh}" = x ; then
   AC_HEADER_CHECK(tk.h, ac_cv_c_tkh=installed, ac_cv_c_tkh="")
fi
])
if test x"${ac_cv_c_tkh}" != x ; then
#  no_tk=""
  if test x"${ac_cv_c_tkh}" = x"installed" ; then
    AC_MSG_RESULT([is installed])
    TKHDIRDASHI=""
  else
    AC_MSG_RESULT([found in ${ac_cv_c_tkh}])
    # this hack is cause the TKHDIRDASHI won't print if there is a "-I" in it.
    TKHDIR="${ac_cv_c_tkh}"
    TKHDIRDASHI="-I${ac_cv_c_tkh}"
  fi
else
  TKHDIRDASHI="# no Tk directory found"
  AC_MSG_WARN([Can't find Tk private headers])
  no_tk=true
fi

AC_SUBST(TKHDIRDASHI)
])

#####
# Once more, this was taken from the Expect 5.25 distribution.
# They have such nice support for Tcl/Tk in configure scripts.

# Defined as a separate macro so we don't have to cache the values
# from PATH_TKCONFIG (because this can also be cached).
AC_DEFUN(CY_AC_LOAD_TKCONFIG, [
    if test -f "$TKCONFIG" ; then
      . $TKCONFIG
    fi

    AC_SUBST(TK_VERSION)
dnl not actually used, don't export to save symbols
dnl    AC_SUBST(TK_MAJOR_VERSION)
dnl    AC_SUBST(TK_MINOR_VERSION)
    AC_SUBST(TK_DEFS)

dnl not used, don't export to save symbols
    dnl AC_SUBST(TK_LIB_FILE)

dnl not used outside of configure
dnl    AC_SUBST(TK_LIBS)
dnl not used, don't export to save symbols
dnl    AC_SUBST(TK_PREFIX)

dnl not used, don't export to save symbols
dnl    AC_SUBST(TK_EXEC_PREFIX)

    AC_SUBST(TK_XINCLUDES)
    AC_SUBST(TK_XLIBSW)
    AC_SUBST(TK_BUILD_LIB_SPEC)
    AC_SUBST(TK_LIB_SPEC)
])

AC_DEFUN(CY_AC_LOAD_TCLCONFIG, [
    . $TCLCONFIG

dnl AC_SUBST(TCL_VERSION)
dnl AC_SUBST(TCL_MAJOR_VERSION)
dnl AC_SUBST(TCL_MINOR_VERSION)
dnl AC_SUBST(TCL_CC)
    AC_SUBST(TCL_DEFS)

dnl not used, don't export to save symbols
dnl    AC_SUBST(TCL_LIB_FILE)

dnl don't export, not used outside of configure
dnl     AC_SUBST(TCL_LIBS)
dnl not used, don't export to save symbols
dnl    AC_SUBST(TCL_PREFIX)

dnl not used, don't export to save symbols
dnl    AC_SUBST(TCL_EXEC_PREFIX)


dnl not used, don't export to save symbols
dnl AC_SUBST(TCL_SHLIB_CFLAGS)
    AC_SUBST(TCL_SHLIB_LD)
dnl don't export, not used outside of configure
dnl AC_SUBST(TCL_SHLIB_LD_LIBS)

# Tcl defines TCL_SHLIB_SUFFIX but TCL_SHARED_LIB_SUFFIX then looks for it
# as just SHLIB_SUFFIX.  How bizarre.
    SHLIB_SUFFIX=$TCL_SHLIB_SUFFIX
    AC_SUBST(SHLIB_SUFFIX)

dnl not used, don't export to save symbols
dnl AC_SUBST(TCL_DL_LIBS)
    AC_SUBST(TCL_LD_FLAGS)
dnl don't export, not used outside of configure
dnl AC_SUBST(TCL_LD_SEARCH_FLAGS)
dnl AC_SUBST(TCL_COMPAT_OBJS)
dnl AC_SUBST(TCL_RANLIB)

# if Tcl's build directory has been removed, TCL_LIB_SPEC should
# be used instead of TCL_BUILD_LIB_SPEC
SAVELIBS=$LIBS
LIBS="$TCL_BUILD_LIB_SPEC $TCL_LIBS"
AC_CHECK_FUNC(Tcl_CreateCommand,[
        AC_MSG_CHECKING([if Tcl library build specification is valid])
        AC_MSG_RESULT(yes)
],[
        TCL_BUILD_LIB_SPEC=$TCL_LIB_SPEC
        # Can't pull the following CHECKING call out since it will be
        # broken up by the CHECK_FUNC just above.
        AC_MSG_CHECKING([if Tcl library build specification is valid])
        AC_MSG_RESULT(no)
])
LIBS=$SAVELIBS

    AC_SUBST(TCL_BUILD_LIB_SPEC)
    AC_SUBST(TCL_LIB_SPEC)
dnl AC_SUBST(TCL_LIB_VERSIONS_OK)

    AC_SUBST(TCL_SHARED_LIB_SUFFIX)

dnl not used, don't export to save symbols
dnl    AC_SUBST(TCL_UNSHARED_LIB_SUFFIX)
])

#####
# Locate the tclConfig.sh and tkConfig.sh files
#
AC_DEFUN(CY_AC_PATH_TCLCONFIG, [
#
# Ok, lets find the tcl configuration
# First, look for one uninstalled.
# the alternative search directory is invoked by --with-tclconfig
#

if test x"${no_tcl}" = x ; then
  # we reset no_tcl in case something fails here
  no_tcl=true
  AC_ARG_WITH(tclconfig, [  --with-tclconfig           directory containing tcl configuration (tclConfig.sh)],
         with_tclconfig=${withval})
  AC_MSG_CHECKING([for Tcl configuration])
  AC_CACHE_VAL(ac_cv_c_tclconfig,[

  # First check to see if --with-tclconfig was specified.
  if test x"${with_tclconfig}" != x ; then
    if test -f "${with_tclconfig}/tclConfig.sh" ; then
      ac_cv_c_tclconfig=`(cd ${with_tclconfig}; pwd)`
    else
      AC_MSG_ERROR([${with_tclconfig} directory doesn't contain tclConfig.sh])
    fi
  fi

  # then check for a private Tcl installation
  if test x"${ac_cv_c_tclconfig}" = x ; then
    for i in \
                ../tcl \
                `ls -dr ../tcl[[7-9]].[[0-9]] 2>/dev/null` \
                ../../tcl \
                `ls -dr ../../tcl[[7-9]].[[0-9]] 2>/dev/null` \
                ../../../tcl \
                `ls -dr ../../../tcl[[7-9]].[[0-9]] 2>/dev/null` ; do
      if test -f "$i/unix/tclConfig.sh" ; then
        ac_cv_c_tclconfig=`(cd $i/unix; pwd)`
        break
      fi
    done
  fi
  # check in a few common install locations
  if test x"${ac_cv_c_tclconfig}" = x ; then
    for i in `ls -d ${prefix}/lib /usr/local/lib 2>/dev/null` ; do
      if test -f "$i/tclConfig.sh" ; then
        ac_cv_c_tclconfig=`(cd $i; pwd)`
        break
      fi
    done
  fi
  # check in a few other private locations
  if test x"${ac_cv_c_tclconfig}" = x ; then
    for i in \
                ${srcdir}/../tcl \
                `ls -dr ${srcdir}/../tcl[[7-9]].[[0-9]] 2>/dev/null` ; do
      if test -f "$i/unix/tclConfig.sh" ; then
        ac_cv_c_tclconfig=`(cd $i/unix; pwd)`
        break
      fi
    done
  fi
  ])
  if test x"${ac_cv_c_tclconfig}" = x ; then
    TCLCONFIG="# no Tcl configs found"
    AC_MSG_WARN(Can't find Tcl configuration definitions)
  else
    no_tcl=
    TCLCONFIG=${ac_cv_c_tclconfig}/tclConfig.sh
    AC_MSG_RESULT(found $TCLCONFIG)
  fi
fi
])

AC_DEFUN(CY_AC_PATH_TKCONFIG, [
#
# Ok, lets find the tk configuration
# First, look for one uninstalled.
# the alternative search directory is invoked by --with-tkconfig
#

if test x"${no_tk}" = x ; then
  # we reset no_tk in case something fails here
  no_tk=true
  AC_ARG_WITH(tkconfig, [  --with-tkconfig           directory containing tk configuration (tkConfig.sh)],
         with_tkconfig=${withval})
  AC_MSG_CHECKING([for Tk configuration])
  AC_CACHE_VAL(ac_cv_c_tkconfig,[

  # First check to see if --with-tkconfig was specified.
  if test x"${with_tkconfig}" != x ; then
    if test -f "${with_tkconfig}/tkConfig.sh" ; then
      ac_cv_c_tkconfig=`(cd ${with_tkconfig}; pwd)`
    else
      AC_MSG_ERROR([${with_tkconfig} directory doesn't contain tkConfig.sh])
    fi
  fi

  # then check for a private Tk library
  if test x"${ac_cv_c_tkconfig}" = x ; then
    for i in \
                ../tk \
                `ls -dr ../tk[[4-9]].[[0-9]] 2>/dev/null` \
                ../../tk \
                `ls -dr ../../tk[[4-9]].[[0-9]] 2>/dev/null` \
                ../../../tk \
                `ls -dr ../../../tk[[4-9]].[[0-9]] 2>/dev/null` ; do
      if test -f "$i/unix/tkConfig.sh" ; then
        ac_cv_c_tkconfig=`(cd $i/unix; pwd)`
        break
      fi
    done
  fi
  # check in a few common install locations
  if test x"${ac_cv_c_tkconfig}" = x ; then
    for i in `ls -d ${prefix}/lib /usr/local/lib 2>/dev/null` ; do
      if test -f "$i/tkConfig.sh" ; then
        ac_cv_c_tkconfig=`(cd $i; pwd)`
        break
      fi
    done
  fi
  # check in a few other private locations
  if test x"${ac_cv_c_tkconfig}" = x ; then
    for i in \
                ${srcdir}/../tk \
                `ls -dr ${srcdir}/../tk[[4-9]].[[0-9]] 2>/dev/null` ; do
      if test -f "$i/unix/tkConfig.sh" ; then
        ac_cv_c_tkconfig=`(cd $i/unix; pwd)`
        break
      fi
    done
  fi
  ])
  if test x"${ac_cv_c_tkconfig}" = x ; then
    TKCONFIG="# no Tk configs found"
    AC_MSG_WARN(Can't find Tk configuration definitions)
  else
    no_tk=
    TKCONFIG=${ac_cv_c_tkconfig}/tkConfig.sh
    AC_MSG_RESULT(found $TKCONFIG)
  fi
fi

])

